import 'dart:async';

import 'package:async_redux/async_redux.dart';
import 'package:flutter/foundation.dart';
import "package:test/test.dart";

// Developed by Marcelo Glasberg (Aug 2019).
// For more info, see: https://pub.dartlang.org/packages/async_redux

/// This tests serve as examples to show details of execution order
/// for both sync and async reducers.
///
/// IMPORTANT: To run the tests, put this file in a test directory.

///////////////////////////////////////////////////////////////////////////////

/// The app state, which in this case is just a text.
class AppState {
  final String text;

  AppState({this.text});

  AppState copy(String text) => AppState(
        text: text ?? this.text,
      );

  static AppState initialState() => AppState(text: 'A');

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is AppState && runtimeType == other.runtimeType && text == other.text;

  @override
  int get hashCode => text.hashCode;

  @override
  String toString() => text.toString();
}

///////////////////////////////////////////////////////////////////////////////

List<AppState> states;

class Action1B extends ReduxAction<AppState> {
  @override
  AppState reduce() {
    states.add(state);
    return state.copy(state.text + 'B');
  }
}

///////////////

class Action2B extends ReduxAction<AppState> {
  @override
  AppState reduce() {
    states.add(state);
    dispatch(Action2C());
    states.add(state);
    return state.copy(state.text + 'B');
  }
}

class Action2C extends ReduxAction<AppState> {
  @override
  AppState reduce() {
    return state.copy(state.text + 'C');
  }
}

///////////////

class Action3B extends ReduxAction<AppState> {
  @override
  AppState reduce() {
    states.add(state);
    print('xxxxxxxx');
    dispatch(Action3C());
    print('yyyyyyyy');
    states.add(state);
    print('2state==$state');
    return state.copy(state.text + 'B');
  }
}

class Action3C extends ReduxAction<AppState> {
  @override
  Future<AppState> reduce() async {
    print('3c1state==$state');
    await Future.sync(() {});
    print('3c2state==$state');
    return state.copy(state.text + 'C');
  }
}

///////////////

class Action4B extends ReduxAction<AppState> {
  @override
  Future<AppState> reduce() async {
    states.add(state);
    print('xxxxxxxx');
    await Future.delayed(Duration(seconds: 2));
    await Future.sync(() {});
    dispatch(Action4C());
    await Future.delayed(Duration(seconds: 2));
    print('yyyyyyyy');
    states.add(state);
    print('2state==$state');
    await Future.delayed(Duration(seconds: 2));
    return state.copy(state.text + 'B');
  }
}

class Action4C extends ReduxAction<AppState> {
  @override
  Future<AppState> reduce() async {
    await Future.delayed(Duration(seconds: 2));
    print('3c1state==$state');
    await Future.sync(() {});

    print('3c2state==$state');
    await Future.delayed(Duration(seconds: 4));
    return state.copy(state.text + 'C');
  }
}

///////////////

///////////////////////////////////////////////////////////////////////////////

void main() {
  /////////////////////////////////////////////////////////////////////////////

//  test(
//      '1) A sync reducer is called, '
//      'and no actions are dispatched inside of the reducer. '
//      'It acts as a pure function, just like a regular reducer of "vanilla" Redux.', () async {
//    //
//    states = [];
//
//    var storeTester = StoreTester<AppState>(initialState: AppState.initialState());
//
//    // - Before the action is dispatched, the text is 'A'.
//    expect(storeTester.state.text, 'A');
//
//    // Sync action.
//    //   states.add(state);
//    //   return state.copy(state.text + 'B');
//    storeTester.dispatch(Action1B());
//    TestInfo<AppState> info = await storeTester.waitAllUnorderedGetLast([Action1B]);
//
//    // - When the reducer starts, the text is 'A'.
//    expect(states, [AppState(text: 'A')]);
//
//    // - After the reducer finishes, the text is 'AB'.
//    expect(info.state.text, 'AB');
//  });
//
//  /////////////////////////////////////////////////////////////////////////////
//
//  test(
//      '2) A sync reducer is called, '
//      'which dispatches another sync action. '
//      'They are both executed synchronously.', () async {
//    //
//    states = [];
//
//    var storeTester = StoreTester<AppState>(initialState: AppState.initialState());
//
//    // - Before the action is dispatched, the text is 'A'.
//    expect(storeTester.state.text, 'A');
//
//    // Sync action:
//    //   states.add(state);
//    //   dispatch(Action2C());
//    //   states.add(state);
//    //   return state.copy(state.text + 'B');
//    storeTester.dispatch(Action2B());
//    TestInfo<AppState> info = await storeTester.waitAllUnorderedGetLast([Action2B, Action2C]);
//
//    // - When the reducer starts, the text is 'A'.
//    // - After Action2C(), the text is 'AC'.
//    expect(states, [AppState(text: 'A'), AppState(text: 'AC')]);
//
//    // - After the reducer finishes, the text is 'ACB'.
//    expect(info.state.text, 'ACB');
//  });
//
//  /////////////////////////////////////////////////////////////////////////////
//
//  test(
//      '3) A sync reducer is called, '
//      'which dispatches an ASYNC action.', () async {
//    //
//    states = [];
//
//    var storeTester = StoreTester<AppState>(initialState: AppState.initialState());
//
//    // - Before the action is dispatched, the text is 'A'.
//    expect(storeTester.state.text, 'A');
//
//    // Sync action:
//    //   states.add(state);
//    //   dispatch(Action3C());
//    //   states.add(state);
//    //   return state.copy(state.text + 'B');
//    storeTester.dispatch(Action3B());
//    TestInfo<AppState> info = await storeTester.waitAllUnorderedGetLast([Action3B, Action3C]);
//
//    // - When the reducer starts, the text is 'A'.
//    // - After Action2C(), the text is 'AC'.
//    expect(states, [AppState(text: 'A'), AppState(text: 'A')]);
//
//    // - After the reducer finishes, the text is 'ACB'.
//    expect(info.state.text, 'ABC');
//  });
//
//  /////////////////////////////////////////////////////////////////////////////

//  test(
//      '4) A sync reducer is called, '
//      'which dispatches an ASYNC action.', () async {
//    //
//    states = [];
//
//    var storeTester = StoreTester<AppState>(initialState: AppState.initialState());
//
//    // - Before the action is dispatched, the text is 'A'.
//    expect(storeTester.state.text, 'A');
//
//    // Sync action:
//    //   states.add(state);
//    //   dispatch(Action3C());
//    //   states.add(state);
//    //   return state.copy(state.text + 'B');
//    storeTester.dispatch(Action4B());
//    TestInfo<AppState> info = await storeTester.waitAllUnorderedGetLast([Action4B, Action4C]);
//
//    // - When the reducer starts, the text is 'A'.
//    // - After Action2C(), the text is 'AC'.
////    expect(states, [AppState(text: 'A'), AppState(text: 'A')]);
//
//    // - After the reducer finishes, the text is 'ACB'.
//    expect(info.state.text, 'ABC');
//  });

  test('X', () async {
    //
    print('ANTES---------------------');
    var a = 0;
    Future.value('x').then((value) {
      print('DURANTE');
      a = 1;
      print(a);
      print(value);
    });

    print('DEPOIS');
    print(a);
  });

  test('y', () async {
    //
    print('ANTES-----------------------------');
    var a = 0;
    SynchronousFuture(Future.value('x')).then((value) {
      print('DURANTE');
      a = 1;
      print(a);
      print(value);
    });

    print('DEPOIS');
    print(a);
  });

  test('z', () {
    //
    print('XANTES-----------------------------------');
    var a = 0;
    SynchronousFuture(Future.delayed(Duration(seconds: 3), () {
      print('SYNC');
      return 'x';
    })).then((value) {
      print('XDURANTE');
      a = 1;
      print(a);
      print(value);
    });

    print('XDEPOIS');
    print(a);
  });

  /////////////////////////////////////////////////////////////////////////////
}
